@@ -222,6 +222,7 @@ EXT_BEGIN_MODULE_GLOBALS(EXT)
   char *hosts, *servlet;
   char *redirect_port; 			/* the port sub-string from hosts */
   int ini_user;
+  short java_socket_inet;		/* copy of java_socket_inet */
 
   /* for user CB's */
   zend_stack *cb_stack;
@@ -233,6 +234,9 @@ EXT_BEGIN_MODULE_GLOBALS(EXT)
    may share these and set (*env)->is_shared=1 */
   int peer, peerr;
   char *servlet_ctx;
+
+  /* Class name cache, used by the java() function */
+  HashTable classNameCache;
 EXT_END_MODULE_GLOBALS(EXT)
 
 
@@ -254,6 +258,7 @@ extern void EXT_GLOBAL(clone_cfg)(TSRMLS_D);
 extern void EXT_GLOBAL(destroy_cloned_cfg)(TSRMLS_D);
 
 extern char* EXT_GLOBAL(test_server)(int *socket, short *is_local, struct sockaddr*saddr TSRMLS_DC);
+extern short EXT_GLOBAL(can_fork)(TSRMLS_D);
 
 /* returns the servlet context or null */
 extern char *EXT_GLOBAL(get_servlet_context)(TSRMLS_D);
