@@ -54,19 +54,6 @@ import php.java.bridge.Util;
  * System.out.println(((Invocable)e).invokeFunction("f", new Object[]{}));<br>
  * e.eval((String)null);<br>
  * </code><br>
- * When using an URLReader, the external PHP script must contain a call() back into our java continuation at the end of the script:<br>
- * <code>
- * &lt;?php java_context->call(java_closure()) ?&gt;<br>
- * </code><br>
- * And, if the C-based PHP/Java Bridge extension is used,  the php.ini must contain a java.servlet setting, so that the bridge selects the servlet- instead of the standalone back-end. Example:<br>
- * <code>
- * extension=java.so<br>
- * ;;on windows use: extension=php_java.dll<br>
- * [java]<br>
- * java.servlet=On<br>
- * </code>
- * @author jostb
- *
  */
 public class InvocablePhpScriptEngine extends SimplePhpScriptEngine implements Invocable {
     private static boolean registeredHook = false;
@@ -88,12 +75,6 @@ public class InvocablePhpScriptEngine extends SimplePhpScriptEngine implements I
         super(factory);
     }
 
-    /* Revert constructor chain. Call super(false); privateInit(); super.initialize(), 
-     * see PhpFacesScriptEngine constructor and PhpScriptEngine() constructor. -- The jsr223 API is really odd ... */
-    protected InvocablePhpScriptEngine(boolean initialize) {
-        super(initialize);
-    }
-
     /* (non-Javadoc)
      * @see javax.script.Invocable#call(java.lang.String, java.lang.Object[])
      */
@@ -152,7 +133,7 @@ public class InvocablePhpScriptEngine extends SimplePhpScriptEngine implements I
      * @see javax.script.Invocable#getInterface(java.lang.Object, java.lang.Class)
      */
     public Object getInterface(Object thiz, Class clasz) {
-      if(thiz==null) throw new NullPointerException("Object thiz cannot be null.");
+      if(thiz==null) throw new NullPointerException("Object thiz cannot be null. Please check if the previous call to eval() reported any errors.");
       return ((PhpProcedureProxy)thiz).getNewFromInterface(clasz);
     }
 
@@ -173,8 +154,7 @@ public class InvocablePhpScriptEngine extends SimplePhpScriptEngine implements I
 
         try {
             /* header: <? require_once("http://localhost:<ourPort>/JavaBridge/java/Java.inc"); ?> */
-            // TODO: enable cache globally and remove this
-            localReader = new StringReader("<?php if(!extension_loaded('java')) {(require_once(\""+ctx.getContextString()+"/java/Java.inc\"));}?>");
+            localReader = new StringReader("<?php require_once(\""+ctx.getContextString()+"/java/Java.inc\"); ?>");
             try { while((c=localReader.read(buf))>0) w.write(buf, 0, c);} catch (IOException e) {throw this.scriptException = new PhpScriptException("Could not read header", e);}
             try { localReader.close(); } catch (IOException e) {throw this.scriptException = new PhpScriptException("Could not close header", e);}
     
@@ -195,9 +175,13 @@ public class InvocablePhpScriptEngine extends SimplePhpScriptEngine implements I
             }
             try { localReader.close(); } catch (IOException e) {throw this.scriptException = new PhpScriptException("Could not evaluate footer", e);}
     
-            /* get the proxy, either the one from the user script or our default proxy */
-            try { this.scriptClosure = this.script.getProxy(new Class[]{}); } catch (Exception e) { return null; }
-            handleRelease();
+            if (this.script!=null) {
+        	/* get the proxy, either the one from the user script or our default proxy */
+        	try { this.scriptClosure = this.script.getProxy(new Class[]{}); } catch (Exception e) { return null; }
+        	handleRelease();
+            } else {
+        	throw this.scriptException = new PhpScriptException("Could not evaluate script, please check your php.ini file and see the error log for details.");
+            }
         } finally {
             if(w!=null)  try { w.close(); } catch (IOException e) {/*ignore*/}
             if(localReader!=null) try { localReader.close(); } catch (IOException e) {/*ignore*/}            
