@@ -24,6 +24,7 @@ package php.java.servlet;
  * OTHER DEALINGS IN THE SOFTWARE.
  */
 
+import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
@@ -38,6 +39,7 @@ import javax.servlet.http.HttpServletResponse;
 
 import php.java.bridge.Util;
 import php.java.bridge.Util.Process;
+import php.java.bridge.http.IContextFactory;
 import php.java.servlet.fastcgi.FastCGIServlet;
 
 /**
@@ -109,7 +111,6 @@ public class PhpCGIServlet extends FastCGIServlet {
 	    }
 	} catch (Throwable t) {Util.printStackTrace(t);}      
 
-	Util.TCP_SOCKETNAME = String.valueOf(CGI_CHANNEL);
 	DOCUMENT_ROOT = getRealPath(context, "");
 	SERVER_SIGNATURE = context.getServerInfo();
     }
@@ -134,7 +135,7 @@ public class PhpCGIServlet extends FastCGIServlet {
      * Adjust the standard tomcat CGI env. CGI only.
      */
     public class CGIEnvironment extends FastCGIServlet.CGIEnvironment {
-    	protected AbstractServletContextFactory sessionFactory;
+    	protected SimpleServletContextFactory sessionFactory;
 	public HttpServletRequest req;
     	
 	protected CGIEnvironment(HttpServletRequest req, HttpServletResponse res, ServletContext context) {
@@ -159,7 +160,7 @@ public class PhpCGIServlet extends FastCGIServlet {
 			buf.append("s:");
 		    buf.append("127.0.0.1");
 		    buf.append(":");
-		    buf.append(this.env.get("SERVER_PORT")); 
+		    buf.append(this.environment.get("SERVER_PORT")); 
 		    buf.append('/');
 		    buf.append(req.getRequestURI());
 		    buf.append("javabridge");
@@ -168,36 +169,37 @@ public class PhpCGIServlet extends FastCGIServlet {
 		else 
 		    override = "";
 
-	        this.env.put("X_JAVABRIDGE_OVERRIDE_HOSTS", override);
+	        this.environment.put("X_JAVABRIDGE_OVERRIDE_HOSTS", override);
 	        // same for fastcgi, which already contains X_JAVABRIDGE_OVERRIDE_HOSTS=/ in its environment
-	        this.env.put("X_JAVABRIDGE_OVERRIDE_HOSTS_REDIRECT", override); 
-	        this.env.put("REDIRECT_STATUS", "200");
-	        this.env.put("SERVER_SOFTWARE", Util.EXTENSION_NAME);
-	        this.env.put("HTTP_HOST", this.env.get("SERVER_NAME")+":"+this.env.get("SERVER_PORT"));
+	        this.environment.put("X_JAVABRIDGE_OVERRIDE_HOSTS_REDIRECT", override); 
+	        this.environment.put("REDIRECT_STATUS", "200");
+	        this.environment.put("SERVER_SOFTWARE", Util.EXTENSION_NAME);
+	        this.environment.put("HTTP_HOST", this.environment.get("SERVER_NAME")+":"+this.environment.get("SERVER_PORT"));
 	        String remotePort = null;
 	        try {
 	            remotePort = String.valueOf(req.getRemotePort());
 	        } catch (Throwable t) {
 	            remotePort = String.valueOf(t);
 	        }
-	        this.env.put("REMOTE_PORT", remotePort);
+	        this.environment.put("REMOTE_PORT", remotePort);
 	        String query = req.getQueryString();
 	        if(query!=null)
-	            this.env.put("REQUEST_URI", nullsToBlanks(req.getRequestURI() + "?" + query));
+	            this.environment.put("REQUEST_URI", nullsToBlanks(req.getRequestURI() + "?" + query));
 	        else
-	            this.env.put("REQUEST_URI", nullsToBlanks(req.getRequestURI()));	          
+	            this.environment.put("REQUEST_URI", nullsToBlanks(req.getRequestURI()));	          
 	        
-	        this.env.put("SERVER_ADDR", req.getServerName());
-	        this.env.put("SERVER_SIGNATURE", SERVER_SIGNATURE);
-	        this.env.put("DOCUMENT_ROOT", DOCUMENT_ROOT);
-	        if(req.isSecure()) this.env.put("HTTPS", "On");
+	        this.environment.put("SERVER_ADDR", req.getServerName());
+	        this.environment.put("SERVER_SIGNATURE", SERVER_SIGNATURE);
+	        this.environment.put("DOCUMENT_ROOT", DOCUMENT_ROOT);
+	        if(req.isSecure()) this.environment.put("HTTPS", "On");
 	        
 	        
 		/* send the session context now, otherwise the client has to 
 		 * call handleRedirectConnection */
 	    	String id = req.getHeader("X_JAVABRIDGE_CONTEXT");
-	    	if(id==null) id = ServletContextFactory.addNew(PhpCGIServlet.this, PhpCGIServlet.this.getServletContext(), req, req, res).getId();
-		this.env.put("X_JAVABRIDGE_CONTEXT", id);
+	    	if(id==null) 
+	    	    id = (ctx=ServletContextFactory.addNew(PhpCGIServlet.this, PhpCGIServlet.this.getServletContext(), req, req, res)).getId();
+	    	this.environment.put("X_JAVABRIDGE_CONTEXT", id);
 	    }
 	    return ret;
 	        	
@@ -244,23 +246,29 @@ public class PhpCGIServlet extends FastCGIServlet {
 	protected HeaderParser(CGIRunner runner) {
 	    this.runner = runner;
     	}
-    	protected void parseHeader(String header) {
+    	public void parseHeader(String header) {
 	    runner.addHeader(header);
     	}
     }
     protected class CGIRunner extends CGIServlet.CGIRunner {
+	protected IContextFactory ctx;
 	
 	protected CGIRunner(CGIServlet.CGIEnvironment env) {
 	    super(env);
+	    ctx = ((CGIEnvironment)env).ctx;
 	}
-        protected void run() throws IOException, ServletException {
+        protected void execute() throws IOException, ServletException {
 	    Process proc = null;
+	    
 	    InputStream natIn = null;
 	    OutputStream natOut = null;
+	    ByteArrayOutputStream natErr = new ByteArrayOutputStream();
+	    
 	    InputStream in = null;
 	    OutputStream out = null;
-    	    try {
-        	proc = Util.ProcessWithErrorHandler.start(new String[]{php, "-d", "allow_url_include=On"}, wd, env, phpTryOtherLocations, preferSystemPhp);
+
+	    try {
+        	proc = Util.ProcessWithErrorHandler.start(new String[]{php, "-d", "allow_url_include=On"}, wd, env, phpTryOtherLocations, preferSystemPhp, natErr);
 
         	byte[] buf = new byte[BUF_SIZE];// headers cannot be larger than this value!
 
@@ -279,7 +287,7 @@ public class PhpCGIServlet extends FastCGIServlet {
          	natIn = proc.getInputStream();
     		out = response.getOutputStream();
 
-    		Util.parseBody(buf, natIn, out, new Util.HeaderParser() {protected void parseHeader(String header) {addHeader(header);}});
+    		Util.parseBody(buf, natIn, out, new Util.HeaderParser() {public void parseHeader(String header) {addHeader(header);}});
 
     		try {
      		    proc.waitFor();
@@ -291,9 +299,17 @@ public class PhpCGIServlet extends FastCGIServlet {
     		if(natIn!=null) try {natIn.close();} catch (IOException e) {/*ignore*/}
     		if(natOut!=null) try {natOut.close();} catch (IOException e) {/*ignore*/}
     		if(proc!=null) try {proc.destroy();} catch (Exception e) {/*ignore*/}
+    		
+    		if (ctx!=null) ctx.release();
+    		ctx = null;
+    	    }
+    	    
+    	    if (proc!=null)
+    	    {
+    	        if(natErr.size()>0) Util.logMessage(natErr.toString());
+    	        try {proc.checkError(); } catch (Util.Process.PhpException e) {throw new ServletException(e);}
     	    }
     	    
-    	    if (proc!=null) try {proc.checkError(); } catch (Util.Process.PhpException e) {throw new ServletException(e);}
         }
     } //class CGIRunner
     
@@ -332,11 +348,11 @@ public class PhpCGIServlet extends FastCGIServlet {
      *  (non-Javadoc)
      * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
      */
-    protected void doGet(HttpServletRequest req, HttpServletResponse res)
+    protected void handle(HttpServletRequest req, HttpServletResponse res, boolean handleInput)
 	throws ServletException, IOException {
     	try {
     	   if(!checkPool(res)) return;
- 	    super.doGet(req, res);
+ 	    super.handle(req, res, handleInput);
     	} catch (IOException e) {
     	    try {res.reset();} catch (Exception ex) {/*ignore*/}
 	    StringBuffer buf = new StringBuffer(getRealPath(getServletConfig().getServletContext(), cgiPathPrefix));
